// physicalobject.h
//
// The Universe contains PhysicalObjects
//
// cre: 2012-02-16
// aut: JGS
//
// (C) 2012 EVOLViA

#pragma once

#include "vector3.h"

class PhysicalObject
{
	double
		d_m;		// mass
	Vector3
		d_x,		// center-of-mass position
		d_v,		// velocity vector
		d_F;		// accumulated force acting on this object

	public:

		virtual ~PhysicalObject()
		{}

		PhysicalObject(double mass, Vector3 const & position, Vector3 const & velocity)
		: d_m(mass), d_x(position), d_v(velocity), d_F(Vector3(0,0,0))
		{}

		// Apply a force to the object. This is cumulative
		void applyForce(Vector3 const &F)
		{
			d_F += F;
		}

		// Take a simulation step: accumulated forces are applied.
		// Position and velocity vectors are updated. Force is reset
		// to zero.
		void step(double dt);

		// Accessors (read only)
		Vector3 const & position() const { return d_x; }
		Vector3 const & velocity() const { return d_v; }
		Vector3 const & force() const { return d_F; }
		double mass() const { return d_m; }
};

