// physics.cc
//
// Physics simulation version 1
//
// cre: 2012-02-15
// aut: JGS
//
// (C) 2012 EVOLViA

#include <iostream>
#include <sstream>
#include <iomanip>
#include <cstdlib>
#include "povrender.h"
#include "universe.h"
#include "vector3.h"
#include "physicalobject.h"
#include "physicalobjecttracker.h"

int main()
{
	std::cout << "Physics Simulation 1\n\n";
	system("mkdir output");
	chdir("output");
	std::string basename = "testscenedtrack";
	int width = 1280; //854 / 2;
	int height = 720; // // 480 / 2;

	Universe universe(0.0001);
	PhysicalObject *ooi;
	universe += new PhysicalObject(500.0, Vector3(0.0, 0.5, 0), Vector3(0, 0, 0));
	universe += new PhysicalObject(50.0, Vector3(2.0, 0.5, 0), Vector3(0.0, 0.0, 0.1));
	universe += (ooi = new PhysicalObject(1.0, Vector3(2.0, 0.75, 0), Vector3(0.1, 0.00, 0.00)));
	universe += new PhysicalObjectTracker(*ooi);
	universe += (ooi = new PhysicalObject(15.0, Vector3(-2.0, 0.25, 0), Vector3(0.0, 0.0, -0.15)));
	universe += new PhysicalObjectTracker(*ooi);

	double t_start = 0.0;
	double t_end = 200.0;
	double t_delta = (t_end - t_start) / 20000.0;
	int numframes = 400;
	double t = t_start;

	for (int idx = 0; idx < numframes; idx++)
	{
		while (t < idx * (t_end - t_start) / numframes)
		{
			universe.step(t_delta);
			t += t_delta;
		}

		std::ostringstream filename;
		filename << basename << std::setfill('0') << std::setw(5) << idx;
		POVRender render(filename.str(), width, height);
		render(universe, t / (t_end - t_start));
	}

	system(("yes | ffmpeg -f image2 -i " + basename + "%05d.png -vcodec libx264 -vpre slow -vpre baseline -b 1500k "+ basename + ".mp4").c_str());
	system(("yes | ffmpeg -f image2 -i " + basename + "%05d.png -vcodec libvpx -f webm -b 1500k " + basename + ".webm").c_str());
	system(("mplayer " + basename + ".mp4 -loop 0").c_str());

	return 0;
}
