// universe.cc
//
// cre: 2012-02-16
// aut: JGS
//
// (C) 2012 EVOLViA

#include "universe.h"
#include "physicalobject.h"
#include "etherealobject.h"
#include "vector3.h"

Universe::~Universe()
{
	std::list<PhysicalObject *>::iterator itPhys = d_physicalObjects.begin();
	while (itPhys != d_physicalObjects.end())
		delete *itPhys++;

	std::list<EtherealObject *>::iterator itEth = d_etherealObjects.begin();
	while (itEth != d_etherealObjects.end())
		delete *itEth++;
}

void Universe::step(double dt)
{
	// Apply forces to physical objects
	std::list<PhysicalObject *>::iterator itOuter = d_physicalObjects.begin();
	while (itOuter != d_physicalObjects.end())
	{
		std::list<PhysicalObject *>::iterator itInner = itOuter;
		itInner++;
		while (itInner != d_physicalObjects.end())
		{
			double d = norm((*itInner)->position() - (*itOuter)->position());
			Vector3 r = ((*itInner)->position() - (*itOuter)->position()) / d;
			Vector3 F = -(d_G * (*itOuter)->mass() * (*itInner)->mass() / (d * d)) * r;

			(*itInner)->applyForce(F);
			(*itOuter)->applyForce(-F);

			itInner++;
		}
		itOuter++;
	}

	// Update first the physical objects, then the ethereal ones
	itOuter = d_physicalObjects.begin();
	while (itOuter != d_physicalObjects.end())
		(*itOuter++)->step(dt);

	std::list<EtherealObject *>::iterator itEth = d_etherealObjects.begin();
	while (itEth != d_etherealObjects.end())
		(*itEth++)->step(dt);
}
