// universe.h
//
// Note: physical objects get updated (step()) before ethereal objects.
//
// cre: 2012-02-15
// aut: JGS
//
// (C) 2012 EVOLViA

#include <list>

class PhysicalObject;
class EtherealObject;

class Universe
{
	std::list<PhysicalObject *>
		d_physicalObjects;

	std::list<EtherealObject *>
		d_etherealObjects;

	// Gravitational constant
	double const
		d_G;

	public:

		~Universe();

		Universe(double G = 6.67384e-11)
		: d_G(G)
		{}

		// Add a new physical object to the universe. The pointer is
		// owned by the Universe.
		Universe & operator += (PhysicalObject * object)
		{
			d_physicalObjects.push_back(object); return *this;
		}

		// Add a new ethereal object to the universe. The pointer is
		// owned by the Universe.
		Universe & operator += (EtherealObject * object)
		{
			d_etherealObjects.push_back(object); return *this;
		}

		// Step first all physical objects, then the ethereal ones.
		void step(double dt);

		// Begin and end iterators to walk through the list of physical objects
		typedef std::list<PhysicalObject *>::const_iterator const_iterator_physical;
		typedef std::list<EtherealObject *>::const_iterator const_iterator_ethereal;

		const_iterator_physical begin_physical() const
		{
			return d_physicalObjects.begin();
		}
		const_iterator_physical end_physical() const
		{
			return d_physicalObjects.end();
		}

		const_iterator_ethereal begin_ethereal() const
		{
			return d_etherealObjects.begin();
		}
		const_iterator_ethereal end_ethereal() const
		{
			return d_etherealObjects.end();
		}

		// returns the size of the universe, as total number of physical and ethereal objects
		size_t size() const
		{
			return d_physicalObjects.size() + d_etherealObjects.size();
		}
};

